require( "scene1" )
require( "scene2" )
require( "scene3" )
require( "fade_grids" )

SceneManager = {}
SceneManager.new = function ()
    
    local this = {}
    

    --=========================================================
    -- 初期設定
    --=========================================================
    this.init = function ( self )
    
        self.sceneList = {
            scene1 = Scene1.new(),
            scene2 = Scene2.new(),
            scene3 = Scene3.new(),
        }
        -- 最初のシーン
        self.currentScene = "scene1"
        self.sceneList[this.currentScene]:enter()

        self.nextScene = ""
        self.fadeProcessor = Fade_grids.new()
        self.fadeObj = nil

    end
    

    
    

    --=========================================================
    -- 更新
    --=========================================================
    this.update = function(self, dt)
        if ( self.nextScene == "" ) then -- フェードアウト中は更新停止とする
        	self.sceneList [ self.currentScene ]:update(dt)
        end
        
        -- フェードがセットされていればフェード処理を更新
        if ( self.fadeObj ) then
            
            self.fadeObj:update()
            
            -- フェード処理の終了判定
            if ( self.fadeObj.phase == "" ) then
                self.fadeObj = nil
            end
        end   
    end
    
    --=========================================================
    -- 描画
    --=========================================================
    this.draw = function(self)
        
        love.graphics.clear(1,1,1,1)
        love.graphics.setColor(1,1,1,1)
        
        self.sceneList [ self.currentScene ]:draw()
        
        -- フェードがセットされていればフェードを描画
        if ( self.fadeObj ) then
            self.fadeObj:draw()
        end      
    end
    

    
    
    

    --=========================================================
    -- マウス入力受付
    --=========================================================
    this.mousereleased = function ( self, x, y, buttonNum )
        
        if ( this.sceneList[this.currentScene].mousereleased ) then
            this.sceneList[this.currentScene]:mousereleased ( x, y, buttonNum )
        end
        
    end
    


    

    --=========================================================
    -- 次のシーンへ遷移
    --=========================================================
    this.changeScene = function ( self, nextSceneName )
        
        self.nextScene = nextSceneName
        
        self.fadeObj = self.fadeProcessor
        
        -- フェードクラス側で然るべきタイミングで呼んでもらうためのシーン遷移処理を設定
        self.fadeObj.changeScene = function ()
            
            self.sceneList [ self.currentScene ]:leave()
            self.sceneList [ self.nextScene ]:enter()
            self.currentScene = self.nextScene
            self.nextScene = ""
            
        end
        
        self.fadeObj:start()
    end
    

    return this
end
